package uk.ac.leeds.ccg.andyt.grids.examples;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.HashSet;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellAbstract;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDouble;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunk64CellMapFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunkAbstractFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunkArrayFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunkJAIFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunkMapFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleChunkRAFFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellDoubleFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellInt;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunk64CellMapFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunkAbstractFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunkArrayFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunkJAIFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunkMapFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntChunkRAFFactory;
import uk.ac.leeds.ccg.andyt.grids.core.Grid2DSquareCellIntFactory;
import uk.ac.leeds.ccg.andyt.grids.core.GridStatistics0;
import uk.ac.leeds.ccg.andyt.grids.core.GridStatistics1;
import uk.ac.leeds.ccg.andyt.grids.core.GridStatisticsAbstract;
import uk.ac.leeds.ccg.andyt.grids.exchange.ImageExporter;
import uk.ac.leeds.ccg.andyt.grids.exchange.ESRIAsciiGridExporter;
import uk.ac.leeds.ccg.andyt.grids.process.Grid2DSquareCellProcessorDEM;
import uk.ac.leeds.ccg.andyt.grids.utilities.Utilities;
import uk.ac.leeds.ccg.andyt.grids.utilities.FileCreator;

public  class Example
        extends Grid2DSquareCellProcessorDEM {
    
    private long time;
    
    /** Creates a new ProcessorDEMRuns */
    public Example() {
        this( FileCreator.createNewFile() );
    }
    
    /**
     * Creates a new ProcessorDEMRuns using specified workspace.
     * WARNING: Files in the specified workspace may get overwritten.
     */
    public Example( File workspace ) {
        super( workspace );
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        File workspace = new File( "c:/work/src/andyt/java/grids/workspace/" );
        Example example = new Example( workspace );
        example.run();
    }
    
    public void run() {
        try {
            time = System.currentTimeMillis();
            boolean handleOutOfMemoryError = true;
            File inputFile = new File( this.workspace.getParent(), "/data/TestData/TestDEM/TestDEM0.asc" );
            //File inputFile = new File( "." + System.getProperty( "file.seperator" ) + "data" + System.getProperty( "file.seperator" ) + "crete_w0.asc" );
            //File inputFile = new File( "C:/Work/Projects/DesertLinks/Data/Crete/dem/srtm/crete_w0.asc" );
            //File inputFile = new File( "./data/crete_w0.asc" );
            int chunkNrows = 512; //64, 128, 256, 512, 1024, 2048
            int chunkNcols = 512; //64, 128, 256, 512, 1024, 2048
            String name = new String( inputFile.getName().substring( 0, ( inputFile.getName().length() - 4 ) ) + "_Grid2DSquareCellInt_chunkNrows(" + chunkNrows + ")_chunkNcols(" + chunkNcols + ")" );
            this.grid2DSquareCellIntFactory.setDirectory( new File( inputFile.getParent() + System.getProperty( "file.seperator" ) + name ) );
            this.grid2DSquareCellIntFactory.setChunkNrows( chunkNrows );
            this.grid2DSquareCellIntFactory.setChunkNcols( chunkNcols );
            this.grid2DSquareCellIntFactory.setGridStatistics( new GridStatistics0() );
            this.grid2DSquareCellIntFactory.setHandleOutOfMemoryError( handleOutOfMemoryError );
            this.grid2DSquareCellDoubleFactory.setChunkNrows( chunkNrows );
            this.grid2DSquareCellDoubleFactory.setChunkNcols( chunkNcols );
            this.grid2DSquareCellDoubleFactory.setGridStatistics( new GridStatistics0() );
            Grid2DSquareCellInt grid2DSquareCellInt = ( Grid2DSquareCellInt ) this.grid2DSquareCellIntFactory.create( inputFile );
            grid2DSquareCellInt.setName( name, handleOutOfMemoryError );
            doSlopeAndAspect(
                    this.grid2DSquareCellDoubleFactory,
                    grid2DSquareCellInt );
            log( "Processing complete in " + Utilities.time( System.currentTimeMillis() - time ), handleOutOfMemoryError );
        } catch ( Exception e ) {
            e.printStackTrace();
        } catch ( Error e ) {
            e.printStackTrace();
        }
    }
    
    /**
     * Calculates the slope and aspect of dem and writes results as asciigrid files
     * @param dem
     */
    public void doSlopeAndAspect(
            Grid2DSquareCellDoubleFactory grid2DSquareCellDoubleFactory,
            Grid2DSquareCellAbstract grid2DSquareCell )
            throws IOException {
        boolean handleOutOfMemoryError = true;
        ImageExporter imageExporter = new ImageExporter();
        File file = new File(
                grid2DSquareCell.getDirectory(handleOutOfMemoryError).getParentFile(),
                grid2DSquareCell.getName( handleOutOfMemoryError ) + ".png" );
//        imageExporter.toGreyScaleImage(
//                grid2DSquareCell,
//                file,
//                "PNG",
//                handleOutOfMemoryError );
        String message = new String(
                "doSlopeAndAspect( Grid2DSquareCellAbstract( " +
                grid2DSquareCell.toString( true ) + " ) )" );
        log( message, handleOutOfMemoryError );
        // Initialisation
        BigDecimal[] dimensions = grid2DSquareCell.getDimensions( handleOutOfMemoryError );
        double cellsize = Double.valueOf( dimensions[ 0 ].toString() ).doubleValue();
        double weightIntersect = 1.0d;
        double weightFactor = 1.0d;
        long gridNrows = grid2DSquareCell.getNrows( handleOutOfMemoryError );
        long gridNcols = grid2DSquareCell.getNcols( handleOutOfMemoryError );
        log( "Initialised kernel parameters: cellize( " + cellsize + " ), weightIntersect( " + weightIntersect + " ), weightFactor( " + weightFactor + " ), gridNcols( " + gridNcols + " ), gridNrows( " + gridNrows + " )" , handleOutOfMemoryError);
        long longGridNrowsMinusOne = gridNrows - 1L;
        long longGridNcolsMinusOne = gridNcols - 1L;
        long longZero = 0L;
        long longOne = 1L;
        long longDistances = Long.MIN_VALUE;
        long long0 = Long.MIN_VALUE;
        
        boolean reinitialiseMemoryReserve = true;
        Grid2DSquareCellDouble dummyGrid = null;
        int distances = Integer.MIN_VALUE;
        double distance = Double.NEGATIVE_INFINITY;
        String logString0 = new String(
                "This string should be at least as long as any message that is " +
                "to be logged for memory handling reasons." +
                "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" +
                "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" +
                "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" +
                "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" +
                "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" +
                "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" +
                "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" +
                "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" +
                "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" +
                "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" );
        String logString = logString0;
        file = new File( logString );
        Grid2DSquareCellDouble[] slopeAndAspect = new Grid2DSquareCellDouble[ 4 ];
        for ( int i = 0; i < slopeAndAspect.length; i ++ ) {
            slopeAndAspect[ i ] = new Grid2DSquareCellDouble();
        }
        ESRIAsciiGridExporter eSRIASciiGridExporter = new ESRIAsciiGridExporter();
        
        // Calculate
        for ( distances = 2; distances <= 4; distances *=2 ) {
            distance = cellsize * ( double ) distances;
            logString = "Distance = " + distance;
            log( logString, handleOutOfMemoryError );
            logString = logString0;
            slopeAndAspect = getSlopeAndAspect(
                    grid2DSquareCell,
                    distance,
                    weightIntersect,
                    weightFactor,
                    grid2DSquareCellDoubleFactory,
                    handleOutOfMemoryError );
            for ( int i = 0; i < slopeAndAspect.length; i ++ ) {
                dummyGrid = ( Grid2DSquareCellDouble ) slopeAndAspect[ i ];
                logString = dummyGrid.toString( handleOutOfMemoryError );
                log( logString, handleOutOfMemoryError );
                logString = logString0;
                // Mask for edge effects
                // mask left
                longDistances = ( long ) ( distances );
                long0 = longDistances - longOne;
                mask(   dummyGrid,
                        longZero,
                        longZero,
                        longGridNrowsMinusOne,
                        long0,
                        handleOutOfMemoryError );
                // mask right
                long0 = gridNcols - longDistances;
                mask(   dummyGrid,
                        longZero,
                        long0,
                        longGridNrowsMinusOne,
                        longGridNcolsMinusOne,
                        handleOutOfMemoryError );
                // mask top
                long0 = longDistances - longOne;
                mask(   dummyGrid,
                        longZero,
                        longZero,
                        long0,
                        longGridNcolsMinusOne,
                        handleOutOfMemoryError );
                // mask bottom
                long0 = gridNrows - longDistances;
                mask(   dummyGrid,
                        long0,
                        longZero,
                        longGridNrowsMinusOne,
                        longGridNcolsMinusOne,
                        handleOutOfMemoryError );
                log(    "Writing Out Results",
                        handleOutOfMemoryError );
//                file = new File(
//                        grid2DSquareCell.getDirectory(handleOutOfMemoryError).getParentFile().getParentFile(),
//                        slopeAndAspect[ i ].getName( handleOutOfMemoryError ) + "_" + distances + ".asc" );
//                eSRIASciiGridExporter.toAsciiFile(
//                        slopeAndAspect[ i ],
//                        file,
//                        handleOutOfMemoryError );
                file = new File(
                        grid2DSquareCell.getDirectory(handleOutOfMemoryError).getParentFile(),
                        slopeAndAspect[ i ].getName( handleOutOfMemoryError ) + "_" + distances + ".png" );
                imageExporter.toGreyScaleImage(
                        slopeAndAspect[ i ],
                        file,
                        "PNG",
                        handleOutOfMemoryError );
            }
        }
    }
    
}
